package com.caimei.service;

import com.caimei.model.ResponseJson;
import com.caimei.model.vo.LogisticsBatchVo;
import com.caimei.model.vo.OrderVo;
import com.github.pagehelper.PageInfo;

import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/3/26
 */
public interface OrderService {
    /**
     * 订单数据
     *
     * @param orderState
     * @param userId
     * @param organizeId
     * @param pageNum
     * @param pageSize
     * @return
     */
    ResponseJson<PageInfo<OrderVo>> getOrderList(Integer orderState, Integer userId, Integer organizeId, Integer pageNum, Integer pageSize);

    /**
     * 订单详情
     *
     * @param orderId
     * @return
     */
    ResponseJson<Map<String, Object>> getOrderDetail(Integer orderId);

    /**
     * 物流信息
     *
     * @param orderId
     * @return
     */
    ResponseJson<List<LogisticsBatchVo>> logistics(Integer orderId);

    /**
     * 确认收货
     *
     * @param orderId
     * @return
     */
    ResponseJson<String> affirmCargo(Integer orderId);

    /**
     * 分享订单
     *
     * @param orderId
     * @return
     */
    ResponseJson<String> shareOrder(Integer orderId);

    /**
     * 分享订单,分享码验证通过
     *
     * @param shareCode
     * @param code
     * @param orderId
     * @param organizeId
     * @return
     */
    ResponseJson<Boolean> verifyShareCode(String shareCode, String code, Integer orderId, Integer organizeId);

    /**
     * 删除订单
     *
     * @param orderId
     * @return
     */
    ResponseJson<String> deleteOrder(Integer orderId);

    /**
     * 取消订单
     *
     * @param orderId
     * @return
     */
    ResponseJson<String> cancelOrder(Integer orderId);

    /**
     * 根据关键词获取订单
     *
     * @param searchWord
     * @param organizeId
     * @param pageNum
     * @param pageSize
     * @return
     */
    ResponseJson<PageInfo<OrderVo>> searchOrder(String searchWord, Integer organizeId, Integer pageNum, Integer pageSize);

    /**
     * 根据组织ID查找历史记录
     *
     * @param organizeId
     * @return
     */
    ResponseJson<List<String>> getSearchHistory(Integer organizeId);

    /**
     * 删除历史记录
     *
     * @param organizeId
     * @return
     */
    ResponseJson<String> deleteSearchHistory(Integer organizeId);
}
