package com.caimei.service;

import com.alibaba.fastjson.JSONObject;
import com.caimei.model.ResponseJson;
import com.caimei.model.dto.PaymentDto;
import com.caimei.model.vo.OrderPayLinkVo;
import org.springframework.http.server.reactive.ServerHttpResponse;

import java.util.Map;

/**
 * Description
 *
 * @author : plf
 * @date : 2020/5/9
 */
public interface PayOrderService {
    /**
     * 获取线上支付开关状态
     *
     * @return
     */
    ResponseJson<Integer> getPayOnLineSwitch();

    /**
     * 收银台数据
     *
     * @param orderId 订单id
     * @return
     */
    ResponseJson<Map<String, Object>> checkoutCounter(Integer orderId);

    /**
     * 线上支付
     *
     * @param payment 支付参数对象
     * @return
     */
    ResponseJson<JSONObject> pay(PaymentDto payment);

    /**
     * 异步回调通知
     */
    String paymentCallback(String data) throws Exception;

    /**
     * 小程序生成链接
     */
    ResponseJson<String> payLink(OrderPayLinkVo orderPayLink);

    /**
     * 判断此次支付是否完成
     *
     * @param orderId           订单id
     * @param paySuccessCounter 付款次数
     * @return
     */
    ResponseJson<String> payWhetherSuccess(Integer orderId, Integer paySuccessCounter);

    /**
     * 查询第三方订单状态
     *
     * @param mbOrderId 平台唯一流水号
     * @return
     */
    ResponseJson<JSONObject> findOrderStatus(String mbOrderId);

    /**
     * 延时分账
     */
    void delayedSplitting();

    /**
     * 分账异步通知
     *
     * @param data
     * @return
     */
    String delayedSplittingCallback(String data);

    /**
     * 重定向跳转页面
     *
     * @param linkLogo
     * @param response
     */
    void jumpPage(String linkLogo, ServerHttpResponse response);

    /**
     * 初始化网银支付数据
     *
     * @param linkLogo
     * @return
     */
    ResponseJson<Map<String, Object>> linkData(String linkLogo);
}
